@@ -1,6 +1,6 @@
 package Bench;
 BEGIN {
-  $Bench::VERSION = '0.02';
+  $Bench::VERSION = '0.03';
 }
 # ABSTRACT: Benchmark running times of Perl code
 
@@ -14,53 +14,56 @@ use Time::HiRes qw/gettimeofday tv_interval/;
 my $bench_called;
 my ($t0, $ti);
 
-sub _set_time0    { $t0 = [gettimeofday] }
+sub _set_start_time {
+    $t0 = [gettimeofday];
+}
 
-sub _set_interval { $ti = tv_interval($t0, [gettimeofday]) }
+sub _set_interval {
+    $ti = tv_interval($t0, [gettimeofday]);
+}
 
 sub import {
-    _set_time0;
+    _set_start_time;
     no strict 'refs';
     my $caller = caller();
     *{"$caller\::bench"} = \&bench;
 }
 
-sub _fmt_sec {
-    my $t = shift;
+sub _fmt_num {
+    my ($num, $unit, $nsig) = @_;
+    $nsig //= 4;
     my $fmt;
 
-    if ($t > 1) {
-        $fmt = "%.3fs";
-    } elsif ($t > 0.1) {
-        $fmt = "%.4fs";
+    my $l = $num ? int(log(abs($num))/log(10)) : 0;
+    if ($l >= $nsig) {
+        $fmt = "%.0f";
+    } elsif ($l < 0) {
+        $fmt = "%.${nsig}f";
     } else {
-        $t *= 1000;
-        if ($t > 0.1) {
-            $fmt = "%.3fms";
-        } elsif ($t > 0.01) {
-            $fmt = "%.4fms";
-        } else {
-            $fmt = "%.5fms";
-        }
+        $fmt = "%.".($nsig-$l-1)."f";
     }
-    sprintf($fmt, $t);
+    #say "D:fmt=$fmt";
+    sprintf($fmt, $num) . ($unit // "");
 }
 
 sub bench($;$) {
-    my $opts;
-    if (ref($_[0]) eq 'CODE') {
-        my $sub = shift;
-        $opts = shift;
-        $opts //= {};
-        $opts = {n=>$opts} if ref($opts) ne 'HASH';
-        $opts->{subs} //= {CODE=>$sub};
-    } elsif (ref($_[0]) eq 'HASH') {
-        $opts = shift;
+    my ($subs0, $opts) = @_;
+    $opts //= {};
+    $opts   = {n=>$opts} if ref($opts) ne 'HASH';
+    my %subs;
+    if (ref($subs0) eq 'CODE') {
+        %subs = (a=>$subs0);
+    } elsif (ref($subs0) eq 'HASH') {
+        %subs = %$subs0;
+    } elsif (ref($subs0) eq 'ARRAY') {
+        my $name = "a";
+        for (@$subs0) { $subs{$name++} = $_ }
     } else {
-        die "Usage: bench(CODEREF, OPTS) or bench(OPTS)";
+        die "Usage: bench(CODE|{a=>CODE,b=>CODE, ...}|[CODE, CODE, ...], ".
+            "{opt=>val, ...})";
     }
     die "Please specify one or more subs"
-        unless $opts->{subs} && keys %{$opts->{subs}};
+        unless keys %subs;
 
     my $use_dumbbench;
     if ($opts->{dumbbench}) {
@@ -77,36 +80,33 @@ sub bench($;$) {
         $opts->{dumbbench_options} //= {};
         my $bench = Dumbbench->new(%{ $opts->{dumbbench_options} });
         $bench->add_instances(
-            map { Dumbbench::Instance::PerlSub->new(code => $opts->{subs}{$_}) }
-                keys %{ $opts->{subs} }
+            map { Dumbbench::Instance::PerlSub->new(code => $subs{$_}) }
+                keys %subs
         );
         $bench->run;
         $bench->report;
 
     } else {
 
-        for my $codename (keys %{ $opts->{subs} }) {
-            my $code = $opts->{subs}{$codename};
+        for my $name (sort keys %subs) {
+            my $code = $subs{$name};
 
             my $n = $opts->{n};
 
-            my $i = 0;
-
             # run code once to set default n & j (to reduce the number of
             # time-interval-taking when n is negative)
-            my $j = 1;
-            _set_time0;
+            my $i = 0;
+            _set_start_time;
             $code->();
             _set_interval;
+            my $j = $ti ? int(1/$ti) : 1000;
             $i++;
-            if ($ti >= 2) {
-                $n //= 1;
+            if ($ti <= 0.01) {
+                $n //= 100;
             } else {
-                $n //= -2;
-                $j = $ti ? int(1/$ti) : 1000;
+                $n //= int(1/$ti);
             }
 
-            _set_time0;
             if ($n >= 0) {
                 while ($i < $n) {
                     $code->();
@@ -126,10 +126,10 @@ sub bench($;$) {
             }
             my $res = join(
                 "",
-                (keys(%{$opts->{subs}}) > 1 ? "$codename: " : ""),
-                sprintf("%d calls (%.0f/s), %s (%s/call)",
-                        $i, $i/$ti, _fmt_sec($ti),
-                        _fmt_sec($i ? $ti/$i : 0))
+                (keys(%subs) > 1 ? "$name: " : ""),
+                sprintf("%d calls (%s/s), %s (%s/call)",
+                        $i, _fmt_num($i/$ti), _fmt_num($ti, "s"),
+                        _fmt_num($ti/$i*1000, "ms"))
             );
             say $res if $void;
             push @res, $res;
@@ -142,8 +142,8 @@ sub bench($;$) {
 }
 
 END {
-    $ti = tv_interval($t0, [gettimeofday]);
-    say _fmt_sec($ti) unless $bench_called || $ENV{HARNESS_ACTIVE};
+    _set_interval;
+    say _fmt_num($ti, "s") unless $bench_called || $ENV{HARNESS_ACTIVE};
 }
 
 1;
@@ -157,17 +157,17 @@ Bench - Benchmark running times of Perl code
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
  # time the whole program
  % perl -MBench -e'...'
- 0.1234s
+ 0.0123s
 
  # basic usage of bench()
  % perl -MBench -e'bench sub { ... }'
- 397 calls (198/s), 2.0054s (0.0051s/call)
+ 100 calls (58548/s), 0.0017s (0.0171ms/call)
 
  # get bench result in a variable
  % perl -MBench -E'my $res = bench sub { ... }'
@@ -175,6 +175,7 @@ version 0.02
  # specify bench options
  % perl -MBench -E'bench sub { ... }, 100'
  % perl -MBench -E'bench sub { ... }, {n=>-5}'
+ 304347 calls (60665/s), 5.017s (0.0165ms/call)
 
  # use Dumbbench as the backend
  % perl -MDumbbench -MBench -E'bench sub { ... }'
@@ -183,9 +184,10 @@ version 0.02
  Rounded run time per iteration: 2.9029e-02 +/- 4.8e-05 (0.2%)
 
  # bench multiple codes
- % perl -MBench -E'bench {subs=>{a=>sub {...}, b=>sub {...}}, n=>-2}'
- a: 397 calls (198/s), 2.0054s (0.0051s/call)
- b: 294 calls (146/s), 2.0094s (0.0068s/call)
+ % perl -MBench -E'bench {a=>sub{...}, b=>sub{...}}, {n=>-2}'
+ % perl -MBench -E'bench [sub{...}, sub{...}]'; # automatically named a, b, ...
+ a: 100 calls (12120/s), 0.0083s (0.0825ms/call)
+ b: 100 calls (5357/s), 0.0187s (0.187ms/call)
 
 =head1 DESCRIPTION
 
@@ -198,19 +200,13 @@ This module can utilize L<Dumbbench> as the backend.
 
 =head1 FUNCTIONS
 
-=head2 bench
-
-Syntax:
-
- bench(CODEREF)                    # bench a single sub, default options
- bench(CODEREF, HASHREF)           # specify options
- bench(CODEREF, INT)               # equivalent to bench(CODEREF, {n=>INT})
- bench(HASHREF)                    # bench multiple subs, must specify 'subs'
+=head2 bench SUB(S)[, OPTS] => RESULT
 
 Run Perl code and time it. Exported by default. Will print the result if called
-in void context.
+in void context. SUB can be a coderef for specifying a single sub, or
+hashref/arrayref for specifying multiple subs.
 
-Options are specified in HASHREF. Available options:
+Options are specified in hashref OPTS. Available options:
 
 =over 4
 
@@ -218,13 +214,7 @@ Options are specified in HASHREF. Available options:
 
 Run the code C<n> times, or if negative, until at least C<n> seconds.
 
-If unspecified, the default behaviour is: if code runs for more than 2 seconds,
-it will only be run once (n=1). Otherwise n=-2.
-
-=item * subs => HASHREF
-
-Specify subroutine(s) to time. You normally need not specify this option, unless
-you want to time several subroutines instead of just one.
+If unspecified, the default behaviour is to run at most 1 second or 100 times.
 
 =item * dumbbench => BOOL
 
